Class: Zephira::Tools::MemoryList
Instance Attribute Summary
Attributes inherited from BaseTool
#agent, #args
Class Method Summary
collapse
Instance Method Summary
collapse
Methods inherited from BaseTool
announces_intent?, #arg, #error_result, #initialize, run, #success_result, #validate
Class Method Details
.description ⇒ Object
11
12
13
|
# File 'lib/zephira/tools/memory_list.rb', line 11
def description
"List all stored memory keys."
end
|
.name ⇒ Object
7
8
9
|
# File 'lib/zephira/tools/memory_list.rb', line 7
def name
"memory_list"
end
|
.parameters ⇒ Object
19
20
21
22
23
24
25
26
27
|
# File 'lib/zephira/tools/memory_list.rb', line 19
def parameters
{
type: "object",
properties: {
intent: {type: "string", description: "Brief summary of intent of the operation, meant to be used for context compaction and presentation to the user. Use active voice (e.g., 'Reading X to do Y')."}
},
required: ["intent"]
}
end
|
.read_only? ⇒ Boolean
15
16
17
|
# File 'lib/zephira/tools/memory_list.rb', line 15
def read_only?
true
end
|
Instance Method Details
#run ⇒ Object
30
31
32
33
34
|
# File 'lib/zephira/tools/memory_list.rb', line 30
def run
keys = MemoryStore.keys
agent.status.verbose(" • Memory list: #{keys.size} keys")
success_result(keys)
end
|